#include "shapes.inc"
#include "OTO.inc"

#declare BuildingTexture =
texture{
  pigment{
    planar
    scale 4
    turbulence 1 lambda 3
    color_map{[0 rgb <1, 0.65, 0.25>][1 rgb <1, 0.7, 0.35>]}
    }
  normal{
    granite -.1 poly_wave .5
    accuracy .001
    }
  }

#declare CeilingTexture = BuildingTexture;


#declare FloorTexture =
  texture{
    pigment{
      granite scale 10 poly_wave .35
      color_map{[0 rgb <1,0.75,0.5>*.5][1 rgb <1,0.75,0.5>]}
      }
    }
  
#declare GroundLevel =
union {
	difference {
		box {<75,0,75>, <-75,7,-75>}
		box {<47.78,-0.1,47.78>, <-47.78,7.1,-47.78>}
		texture {BuildingTexture}
		}

	#local idx = 0;
	#while (idx < 360)
		cylinder { <-47.785,0,0>, <47.785,0,0>, 0.02
			translate <0,6.98,47.8> rotate y*idx texture {BuildingTexture}
			}
		#local idx = idx + 90;
	#end

	// the court yard floor
	#declare OTOScale = 1.5; /* since the macro is rudimentary, here's an ad-hoc solution to change the tile size */
	object{ OTO( <3/OTOScale,-48/OTOScale,-.05/OTOScale>, <48/OTOScale,48/OTOScale,1/OTOScale>, 0.3/OTOScale, FloorTexture ) scale OTOScale rotate x*90 }

	// the ceiling
	//difference {
		box {<0,0,47.78>, <-47.78,0,-47.78> //}
		//box {<-5,-0.1,-40>, <5,0.1,40> translate -x*40}
		translate y*5.9
		texture {CeilingTexture}
		}
	
	// the sidewalk
	box{<-47,-1,-47>,<3,.1,47> texture {BuildingTexture}}
	};

#declare ArchSegment =
difference {
	// the archway
	union {
		// edge rounding
		#local Rd = 0.1;
		
		// rounded arch
		difference {
			box {<1,0,7>, <-1,6,-7> }
			cylinder {<-1.1,0.0>, <1.1,0,0>, 5+Rd }
			}
		difference {
			box {<1-Rd,0,7>, <-1+Rd,6,-7> }
			cylinder {<-1.1,0.0>, <1.1,0,0>, 5 }
			}

		difference {
			torus {5+Rd, Rd rotate z*90 translate -x*(1-Rd)}
			torus {5+Rd, Rd rotate z*90 translate x*(1-Rd)}
			box {<1.5,0,10>, <-1.5,-6,-10>}
			}
		
		// edge rounding
		#local Rd = 0.05;
		
		// the arch support
		object {Round_Box_Union (<1.5,0,1.5>, <-1.5,12,-1.5>, Rd) translate z*7}
		
		// moulding thickness
		#local Mthk = .1;
		
		// top moulding
		object {Round_Box_Union (<1+Mthk,5.5,7>, <-1-Mthk,6,-7>, Rd) }
		
		// arch moulding
    object {Round_Box_Union (<1+Mthk,0,2+Mthk>, <-1-Mthk,.35,-2-Mthk>, Rd) translate z*7}
    
    // arch support moulding
    object {Round_Box_Union (<1.5+Mthk,0,1+Mthk>, <-1.5-Mthk,.35,-1.5-Mthk>, Rd) translate z*7}
    object {Round_Box_Union (<1.5+Mthk,5.5,1.5+Mthk>, <-1.5-Mthk,6,-1.5-Mthk>, Rd) translate z*7}
    
		// arch support cap		
		#local idx = 0;
		difference {
			box {<1.75,0,1.75>, <-1.75,2,-1.75> }
			#while (idx < 360)
				cylinder {<2,0,0>, <-2,0,0>, 0.25 translate -z*1.75 rotate y*idx }
				#local idx = idx + 90;
			#end
			translate <0,12,7>
			}
		}

	// z axis walkway
	union {
		box {<0.9,0,8.6>, <-0.9,3.1,-8.6> }
		cylinder {<0,0,8.6>, <0,0,-8.6>, 0.9 translate y*3.1}
		translate y*5.9
		}

	// x axis walkway opening
	union {
		box {<0.9,0,8.6>, <-0.9,3.1,-8.6> }
		cylinder {<0,0,8.6>, <0,0,-8.6>, 0.9 translate y*3.1}
		rotate y*90
		translate <0,5.8999999,7>
		}
	translate z*7
	};

#declare SmlArches =
#local idx = 0;
union {
	#while (idx < 50)
		object { ArchSegment scale 0.35715 rotate y*90 translate <idx,0,0> }
		#local idx = idx + 5;
	#end
	};

#macro Step (Depth)
union {
	box {<1,Depth,0>, <-1, 0.05, -0.1>}
	cylinder {<-1,0,0>, <1,0,0>, 0.002 translate <0,0.05,-0.1>}
	}
#end
	
#declare Landing =
box { <-1,0,0>, <1,0.05,-2> }

#declare StairCase =
	union {
		#local zdx = -14;
		#local ydx = 0;
		union {
			#while (zdx < 0)
				object { Step ((ydx+1)*-1) translate <0,ydx,zdx> }
				#local zdx = zdx + 0.1;
				#local ydx = ydx + 0.05;
			#end
			translate <0,0,-1.9>
			}
		
		#local zdx = -14;
		#local ydx = 0;
		union {
			#while (zdx < 0)
				object { Step ((ydx+1)*-1) translate <0,ydx,zdx> }
				#local zdx = zdx + 0.1;
				#local ydx = ydx + 0.05;
			#end
			rotate y*270
			translate <1,0,-1>
			}

		object { Landing translate <0,7,0> }
		texture { BuildingTexture }
		translate x*1
		}

#declare FirstLevel =
union {
	object { GroundLevel }
	object { ArchSegment }
	object { ArchSegment translate z*15.5 }
	object { ArchSegment translate z*31 }
	object { ArchSegment translate -z*15.5 }
	object { ArchSegment translate -z*31 }
	object { ArchSegment translate -z*46.5 }
	object { ArchSegment translate -z*62 }
	object { StairCase rotate y*90 translate <47.78,-0.05,47.78> }
	texture {BuildingTexture}
	};

#declare SecondLevelFloor =
union {
	box { <-0.4,0,-0.4>, <50.75,0.01,25> }
	box { <49.5,0,25>, <75,0.01,-50.5> }
	texture {FloorTexture}
	};

#declare ThirdLevelCeiling =
union {
	box { <-0.6,0,-0.6>, <49.4,1,25> }
	cylinder { <-0.6,0,0>, <49.5,0,0>, 0.1 translate -z*0.6 }
	cylinder { <-0.6,0.5,0>, <49.5,0.5,0>, 0.05 translate -z*0.6 }
	cylinder { <-0.6,1,0>, <49.5,1,0>, 0.1 translate -z*0.6 }

	box { <49.4,0,25>, <75,1,-50.6> }
	cylinder { <49.4,0,0.6>, <49.4,0,-50.6>, 0.1 }
	cylinder { <49.4,0.5,0.6>, <49.4,0.5,-50.6>, 0.05 }
	cylinder { <49.4,1,0.6>, <49.4,1,-50.6>, 0.1 }
	texture {CeilingTexture}
	};

#declare SecondLevel =
union {
	object { SecondLevelFloor }
	object { ArchSegment scale 0.35715 rotate y*180 translate z*5 }
	object { SmlArches }
	object { SmlArches translate <0,0,0> rotate y*90 translate x*50 }

	box { <-0.5,0,0>, <-0.4,5,25> }
	box { <-0.5,0,24.9>, <75,5,25> }
	box { <75,0,25>, <74.9,5,-50.5> }
	box { <75,0,-50.5>, <49.5,5,-50.6> }
	texture { BuildingTexture }
	};

#declare ThirdLevel =
	union {
	object { ArchSegment scale 0.35715 rotate y*180 translate z*5 }
	object { SmlArches }
	object { SmlArches translate <0,0,0> rotate y*90 translate x*50 }
	object { ThirdLevelCeiling translate y*5 }

	box { <-0.5,0,0>, <-0.4,5,25> }
	box { <-0.5,0,24.9>, <75,5,25> }
	box { <75,0,25>, <74.9,5,-50.5> }
	box { <75,0,-50.5>, <49.5,5,-50.6> }
	texture { BuildingTexture }
	};
